using System;
using System.IO;
using NUnit.Framework;
using SmallMvvm.Core.Datatypes;

namespace SmallMvvm.Core.UnitTests.Datatypes._StringExtensions
{
    [TestFixture]
    public class DecodeFromBase64Fixture
    {
        [Test]
        public void Usage()
        {
            string str = "SGVsbG8gV29ybGQh";
            str = str.DecodeFromBase64();

            Assert.That(str, Is.EqualTo("Hello World!"));
        }

        [Test]
        public void UsageDecodeBinaryContent()
        {
            const string CONTENT = "Qk10AAAAAAAAADYAAAAoAAAABQAAAAUAAAABABAAAAAAAD4AAADvv70AAADvv" +
                                   "70AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
                                   "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";

            const string FILE_NAME = ".\\SomeFile2.bin";

            if (File.Exists(FILE_NAME))
                File.Delete(FILE_NAME);

            using (StreamWriter writer = new StreamWriter(FILE_NAME, false))
            {
                writer.Write(CONTENT.DecodeFromBase64());
            }

            string origContent;
            using (StreamReader reader = new StreamReader(".\\Testfiles\\SomeFile.bin"))
            {
                origContent = reader.ReadToEnd();
            }

            string fakeContent;
            using (StreamReader reader = new StreamReader(FILE_NAME))
            {
                fakeContent = reader.ReadToEnd();
            }

            Assert.That(origContent, Is.EqualTo(fakeContent));
        }

        [Test, SetUICulture("en-US")]
        public void WillThrowArgumentException()
        {
            ArgumentException exception = Assert.Throws<ArgumentException>(() => "".DecodeFromBase64("blah"));

            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentException)));
            Assert.That(exception.Message, Is.EqualTo("'blah' is not a supported encoding name.\r\nParameter name: name"));
        }

        [Test, SetUICulture("en-US")]
        public void WillThrowArgumentNullException()
        {
            string str = null;
            ArgumentNullException exception = Assert.Throws<ArgumentNullException>(() => str.DecodeFromBase64());

            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentNullException)));
            Assert.That(exception.Message, Is.EqualTo("Value cannot be null.\r\nParameter name: InString"));
        }
    }
}