using System;
using System.IO;
using NUnit.Framework;
using SmallMvvm.Core.Datatypes;

namespace SmallMvvm.Core.UnitTests.Datatypes._StringExtensions
{
    [TestFixture]
    public class EncodeToBase64Fixture
    {
        [Test]
        public void Usage()
        {
            string str = "Hello World!";
            str = str.EncodeToBase64();

            Assert.That(str, Is.EqualTo("SGVsbG8gV29ybGQh"));
        }

        [Test]
        public void UsageEncodeBinaryContent()
        {
            const string EXPECTED_RESULT = "Qk10AAAAAAAAADYAAAAoAAAABQAAAAUAAAABABAAAAAAAD4AAADvv70AAADvv" +
                                           "70AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
                                           "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";

            string content;
            using (StreamReader reader = new StreamReader(".\\Testfiles\\SomeFile.bin"))
            {
                string tmp = reader.ReadToEnd();
                content = tmp.EncodeToBase64();
            }

            Assert.That(content, Is.EqualTo(EXPECTED_RESULT));
        }

        [Test, SetUICulture("en-US")]
        public void WillThrowArgumentException()
        {
            ArgumentException exception = Assert.Throws<ArgumentException>(() => "".EncodeToBase64("blah"));

            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentException)));
            Assert.That(exception.Message, Is.EqualTo("'blah' is not a supported encoding name.\r\nParameter name: name"));
        }

        [Test, SetUICulture("en-US")]
        public void WillThrowArgumentNullException()
        {
            string str = null;
            ArgumentNullException exception = Assert.Throws<ArgumentNullException>(() => str.EncodeToBase64());

            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentNullException)));
            Assert.That(exception.Message, Is.EqualTo("String reference not set to an instance of a String.\r\nParameter name: s"));
        }
    }
}