﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq.Expressions;
using NUnit.Framework;

namespace SmallMvvm.Core.UnitTests.Property._PropertyChangedBase
{
    [TestFixture]
    public class MultipleOnPropertyChangedFixture
    {
        [Test]
        public void Usage()
        {
            List<PropertyChangedEventArgs> results = new List<PropertyChangedEventArgs>();
            PropertyChangedTestClass instance = new PropertyChangedTestClass();
            instance.PropertyChanged += (sender, args) => results.Add(args);

            Assert.That(results.Count, Is.EqualTo(0));
            instance.MultipleOnPropertyChanged(new[] { "SomeProperty1", "SomeProperty2" });

            Assert.That(results.Count, Is.EqualTo(2));
            Assert.That(results[0].PropertyName, Is.EqualTo("SomeProperty1"));
            Assert.That(results[1].PropertyName, Is.EqualTo("SomeProperty2"));  
        }

        [Test]
        public void UsageWithExpression()
        {
            string str1 = string.Empty;
            string str2 = string.Empty;

            List<PropertyChangedEventArgs> results = new List<PropertyChangedEventArgs>();
            PropertyChangedTestClass instance = new PropertyChangedTestClass();
            instance.PropertyChanged += (sender, args) => results.Add(args);

            Assert.That(results.Count, Is.EqualTo(0));
            instance.MultipleOnPropertyChanged(new[]
                {
                    (Expression<Func<string>>) (() => str1), 
                    (Expression<Func<string>>) (() => str2)
                });

            Assert.That(results.Count, Is.EqualTo(2));
            Assert.That(results[0].PropertyName, Is.EqualTo("str1"));
            Assert.That(results[1].PropertyName, Is.EqualTo("str2"));  
        }
    }
}