﻿using System.ComponentModel;
using NUnit.Framework;

namespace SmallMvvm.Core.UnitTests.Property._PropertyChangedBase
{
    [TestFixture]
    public class OnPropertyChangedFixture
    {

        [Test]
        public void Usage()
        {
            PropertyChangedEventArgs result = null;
            PropertyChangedTestClass instance = new PropertyChangedTestClass();
            instance.PropertyChanged += (sender, args) => result = args;

            Assert.That(result, Is.Null);
            instance.OnPropertyChanged("SomeProperty");

            Assert.That(result, Is.Not.Null);
            Assert.That(result.PropertyName, Is.EqualTo("SomeProperty"));
        }

        #if NET45

        private class NestedNet45PropertyChangedTestClass : PropertyChangedBase
        {
            private string _someProperty;
            public string SomeProperty
            {
                get { return _someProperty; }
                set
                {
                    _someProperty = value;
                    OnPropertyChanged();
                }
            }
        }

        [Test]
        public void Net45Usage()
        {
            PropertyChangedEventArgs result = null;
            NestedNet45PropertyChangedTestClass instance = new NestedNet45PropertyChangedTestClass();
            instance.PropertyChanged += (sender, args) => result = args;

            Assert.That(result, Is.Null);
            instance.SomeProperty = "some value";

            Assert.That(result, Is.Not.Null);
            Assert.That(result.PropertyName, Is.EqualTo("SomeProperty"));  
        }
        
        #endif

        [Test]
        public void UsageWithExpression()
        {
            PropertyChangedEventArgs result = null;
            PropertyChangedTestClass instance = new PropertyChangedTestClass();
            instance.PropertyChanged += (sender, args) => result = args;

            Assert.That(result, Is.Null);
            instance.OnPropertyChanged(() => result);

            Assert.That(result, Is.Not.Null);
            Assert.That(result.PropertyName, Is.EqualTo("result"));
        }
    }
}