﻿using System;
using System.Collections.Generic;
using System.Linq.Expressions;

namespace SmallMvvm.Core.Property
{
    public abstract class ExtendedPropertyBase : PropertyChangedBase, IDisposable
    {
        private readonly Dictionary<string, object> _propertyValues = new Dictionary<string, object>();

        #region Get

        #if !NET20 && !NET30

        protected T Get<T>(Expression<Func<T>> expression, T defaultValue = default(T))
        {
            return Get(GetPropertyName(expression), defaultValue);
        }

        protected T Get<T>(string propertyName, T defaultValue = default(T))
        {
            return _propertyValues.ContainsKey(propertyName) ? (T)_propertyValues[propertyName] : defaultValue;
        }

        #endif

        #endregion

        #region Set

        #if !NET20 && !NET30

        protected void Set<T>(Expression<Func<T>> expression, T value)
        {
            Set(GetPropertyName(expression), value);
        }

        protected void Set<T>(string propertyName, T value)
        {
            if (_propertyValues.ContainsKey(propertyName))
            {
                if (_propertyValues[propertyName] != null &&
                    (_propertyValues[propertyName] ?? new object()).Equals(value))
                    return;

                if (value is IComparable)
                {
                    if (_propertyValues[propertyName] != ((IComparable)value))
                    {
                        _propertyValues[propertyName] = value;
                        OnPropertyChanged(propertyName);
                    }
                }
                else
                {
                    _propertyValues[propertyName] = value;
                    OnPropertyChanged(propertyName);
                }
            }
            else
            {
                _propertyValues.Add(propertyName, value);
                OnPropertyChanged(propertyName);
            }
        }

        #endif

        #if NET45
 
        protected bool SetProperty<T>(ref T storage, T value, [CallerMemberName] string propertyName = null)
        {
            if (Equals(storage, value)) 
                return false;
 
            storage = value;
            OnPropertyChanged(propertyName);
 
            return true;
        }
 
        #endif

        #endregion

        public void Dispose()
        {
            _propertyValues.Clear();
        }
    }
}