﻿using System;
using System.Threading;
using SmallMvvm.Core.Property;

namespace SmallMvvm.Presentation
{
    public abstract class ViewModel : ExtendedPropertyBase
    {
        #region Default context

        public void SendToUiThread(Action action, object stateParams = null)
        {
            uiContext.Send(state => action.Invoke(), stateParams);
        }

        public void PostToUiThread(Action action, object stateParams = null)
        {
            uiContext.Post(state => action.Invoke(), stateParams);
        }

        #endregion

        #region Custom context

        public void SendToThread(Action action, SynchronizationContext context, object stateParams = null)
        {
            context.Send(state => action.Invoke(), stateParams);
        }

        public void PostToThread(Action action, SynchronizationContext context, object stateParams = null)
        {
            context.Post(state => action.Invoke(), stateParams);
        }

        #endregion
    }
}