﻿using System;
using NUnit.Framework;

namespace SmallMvvm.Services.UnitTests._ServiceProvider
{
    [TestFixture]
    public class RegisterServiceFixture
    {
        [Test]
        public void RegisterUsage()
        {
            IServiceProvider serviceProvider = new ServiceProvider();
            serviceProvider.Register<ITestServiceA>(new TestServiceA());
            serviceProvider.Register<ITestServiceB>(new TestServiceB());
            serviceProvider.Register<ITestServiceC>(new TestServiceC());

            ITestServiceA testServiceA = serviceProvider.GetService<ITestServiceA>();
            ITestServiceB testServiceB = serviceProvider.GetService<ITestServiceB>();
            ITestServiceC testServiceC = serviceProvider.GetService<ITestServiceC>();

            Assert.That(testServiceA.Name, Is.EqualTo("Service A"));  
            Assert.That(testServiceB.Name, Is.EqualTo("Service B"));  
            Assert.That(testServiceC.Name, Is.EqualTo("Service C"));  
        }

        [Test, SetUICulture("en-US")]
        public void FailToRegisterExistingService()
        {
            IServiceProvider serviceProvider = new ServiceProvider();
            serviceProvider.Register<IService>(new TestServiceA());

            ArgumentException exception = Assert.Throws<ArgumentException>(() => serviceProvider.Register<IService>(new TestServiceB()));
            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentException)));
            Assert.That(exception.Message.Contains("Service of submitted type already exists. Use unregister to remove existing service type registration!"), Is.True);
        }
        
        [Test, SetUICulture("en-US")]
        public void FailToRegisterNullInstance()
        {
            IServiceProvider serviceProvider = new ServiceProvider();
            ArgumentNullException exception = Assert.Throws<ArgumentNullException>(() => serviceProvider.Register<IService>(null));

            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentNullException)));
            Assert.That(exception.Message.Contains("Service instance cannot be null!"), Is.True);
        }
        
        [Test, SetUICulture("en-US")]
        public void FailToRegisterEmptyServiceName()
        {
            IServiceProvider serviceProvider = new ServiceProvider();
            ArgumentException exception = Assert.Throws<ArgumentException>(() => serviceProvider.Register<IService>(new TestServiceA(), string.Empty));

            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentException)));
            Assert.That(exception.Message.Contains("ServiceName should not be empty!"), Is.True);
        }
        
        [Test, SetUICulture("en-US")]
        public void FailToRegisterSameServiceNameAsServiceType()
        {
            IServiceProvider serviceProvider = new ServiceProvider();
            ArgumentException exception = Assert.Throws<ArgumentException>(() => serviceProvider.Register<IService>(new TestServiceA(), typeof(IService).FullName));

            Assert.That(exception.GetType(), Is.EqualTo(typeof(ArgumentException)));
            Assert.That(exception.Message.Contains("ServiceName should not be the name of the registered type!"), Is.True);
        }
    }
}